//
//  Yazzem.h
//  yazzem2
//
//  Created by Mike Cohen on 2/19/10.
//  Copyright 2010 MC Development. All rights reserved.
//

#if TARGET_OS_IPHONE
// for the iPhone, we use TouchXML
#import "TouchXML.h"
#define NSXMLDocument           CXMLDocument
#define NSXMLNode               CXMLNode
#define NSXMLElement            CXMLElement
#define NSXMLDocumentTidyXML    CXMLDocumentTidyXML
#define NSXMLDocumentTidyHTML   CXMLDocumentTidyHTML
#else
#import <Cocoa/Cocoa.h>
#endif

@interface YazzemUser : NSObject {
    NSString *_user_id;
    NSString *_username;
    NSString *_full_name;
    NSString *_bio;
    NSString *_website;
    NSString *_location;
    NSString *_profile_photo;
    NSString *_reg_date;
}

- (id)initWithElement: (NSXMLElement*)element;
+ (YazzemUser*)UserWithElement: (NSXMLElement*)element;

@property (retain,nonatomic) NSString *user_id;
@property (retain,nonatomic) NSString *username;
@property (retain,nonatomic) NSString *full_name;
@property (retain,nonatomic) NSString *bio;
@property (retain,nonatomic) NSString *website;
@property (retain,nonatomic) NSString *location;
@property (retain,nonatomic) NSString *profile_photo;
@property (retain,nonatomic) NSString *reg_date;

@end

@interface YazzemStatus : NSObject {
    NSString *_text;
    NSString *_status_id;
    NSString *_from;
    NSString *_type;
    NSString *_date;
    YazzemUser *_user;
}

- (id)initWithElement: (NSXMLElement*)element;
+ (YazzemStatus*)StatusWithElement: (NSXMLElement*)element;

@property (retain,nonatomic) NSString *text;
@property (retain,nonatomic) NSString *status_id;
@property (retain,nonatomic) NSString *from;
@property (retain,nonatomic) NSString *type;
@property (retain,nonatomic) NSString *date;
@property (retain,nonatomic) YazzemUser *user;

@end

@interface YazzemTopic : NSObject {
    NSString *_topic_name;
    NSString *_topic_id;
    NSString *_type;
    NSString *_description;
    NSString *_keywords;
    NSString *_date;
    YazzemUser *_user;
}

- (id)initWithElement: (NSXMLElement*)element;
+ (YazzemTopic*)TopicWithElement: (NSXMLElement*)element;

@property (retain,nonatomic) NSString *topic_name;
@property (retain,nonatomic) NSString *topic_id;
@property (retain,nonatomic) NSString *type;
@property (retain,nonatomic) NSString *description;
@property (retain,nonatomic) NSString *keywords;
@property (retain,nonatomic) NSString *date;
@property (retain,nonatomic) YazzemUser *user;

@end;

@interface YazzemMessage : NSObject {
    NSString *_message_id;
    NSString *_text;
    NSString *_subject;
    NSString *_status;
    YazzemUser *_sender;
    YazzemUser *_sentto;
}

- (id)initWithElement: (NSXMLElement*)element;
+ (YazzemMessage*)MessageWithElement: (NSXMLElement*)element;

@property (retain,nonatomic) NSString *message_id;
@property (retain,nonatomic) NSString *text;
@property (retain,nonatomic) NSString *subject;
@property (retain,nonatomic) NSString *status;
@property (retain,nonatomic) YazzemUser *sender;
@property (retain,nonatomic) YazzemUser *sentto;

@end

@interface YazzemComment : NSObject
{
    NSString *_comment_id;
    NSString *_text;
    NSString *_date;
    YazzemUser *_user;
}

- (id)initWithElement: (NSXMLElement*)element;
+ (YazzemComment*)CommentWithElement: (NSXMLElement*)element;


@property (retain,nonatomic) NSString *comment_id;
@property (retain,nonatomic) NSString *text;
@property (retain,nonatomic) NSString *date;
@property (retain,nonatomic) YazzemUser *user;


@end


@interface Yazzem : NSObject {
    id _delegate;

    //NSURLConnection *_connection;
    NSString *_appname;

    NSString *_endpoint;
    NSString *_username;
    NSString *_password;
    
    NSString *_method;
    NSString *_params;
}

//@property (retain) NSURLConnection *connection;

@property (assign,nonatomic) id delegate;

@property (retain,nonatomic) NSString *appname;

@property (retain,nonatomic) NSString *endpoint;
@property (retain,nonatomic) NSString *username;
@property (retain,nonatomic) NSString *password;

@property (retain,nonatomic) NSString *method;
@property (retain,nonatomic) NSString *params;

+ (Yazzem*)Shared;

- (id) init;


- (NSMutableArray*)get_status:(NSString*)inMethod count: (int)count;
- (NSMutableArray*)all_status: (int)count;
- (NSMutableArray*)status_status: (int)count;
- (NSMutableArray*)status_topic: (int)count;
- (NSMutableArray*)get_timeline: (int)count;

- (NSMutableArray*)get_status_comments: (NSString*)status_id count: (int)count;

- (YazzemTopic*)get_topic:(NSString*)topic_id;
- (NSMutableArray*)get_topic_comments: (NSString*)topic_id count: (int)count;
- (NSMutableArray*)get_topic_list: (int)count;

- (YazzemUser*)get_user: (NSString*)user_id;
- (NSMutableArray*)get_user_topics: (NSString*)user_id count:(int)count;
- (NSMutableArray*)get_user_joined: (NSString*)user_id count:(int)count;
- (NSMutableArray*)get_user_status: (NSString*)user_id count:(int)count;
- (NSMutableArray*)get_profile_comments: (NSString*)user_id count:(int)count;
- (NSMutableArray*)get_user_subscribers: (NSString*)user_id count:(int)count;
- (NSMutableArray*)get_user_subscriptions: (NSString*)user_id count:(int)count;

- (NSMutableArray*)get_messages:(int)count;
- (NSMutableArray*)get_messages_read:(int)count;
- (NSMutableArray*)get_messages_unread:(int)count;
- (NSMutableArray*)get_messages_sent: (int)count;

- (NSString*)get_userID: (NSString*)name;


@end

@interface Yazzem (Posting) 

- (YazzemStatus*) status_new: (NSString*)text;
- (YazzemComment*) status_comment_new: (NSString*)status_id comment: (NSString*)text;

- (YazzemTopic*) topic_create: (NSString*)name 
                  description: (NSString*)description 
                     keywords: (NSString*)keywords
                    is_public: (BOOL)is_public;

- (YazzemTopic*) topic_join: (NSString*)topic_id;
- (YazzemTopic*) topic_leave: (NSString*)topic_id;
- (YazzemComment*) topic_comment_new: (NSString*)topic_id comment: (NSString*)text;

- (YazzemUser*) subscribe: (NSString*)user_id;
- (YazzemUser*) unsubscribe: (NSString*)user_id;
- (YazzemComment*) profile_comment_new: (NSString*)user_id comment: (NSString*)text;

- (YazzemMessage*) message_new: (NSString*)to_id
                       subject: (NSString*)subject
                       message: (NSString*)message
                      is_reply: (BOOL)is_reply;

- (void) message_read: (NSString*)message_id;
- (void) message_unread: (NSString*)message_id;

@end
