//
//  Yazzem.m
//  yazzem2
//
//  Created by Mike Cohen on 2/19/10.
//  Copyright 2010 MC Development. All rights reserved.
//

#import "Yazzem.h"

@interface Yazzem(Private)

- (NSXMLElement*)http_post: (NSMutableDictionary*)postdata;
- (NSData*)http_get;
- (NSXMLDocument*)GetXmlResponse;

- (NSMutableArray*)MakeArrayOfTopics: (NSXMLElement*)root named: (NSString*)name;
- (NSMutableArray*)MakeArrayOfStatuses: (NSXMLElement*)root named: (NSString*)name;
- (NSMutableArray*)MakeArrayOfComments: (NSXMLElement*)root named: (NSString*)name;
- (NSMutableArray*)MakeArrayOfMessages: (NSXMLElement*)root named: (NSString*)name;
- (NSMutableArray*)MakeArrayOfUsers: (NSXMLElement*)root named: (NSString*)name;

@end

static NSXMLElement *get_child_element(NSXMLElement *element,NSString* name)
{
    NSArray *children = [element elementsForName: name];
    if ([children count] > 0) {
        return [children objectAtIndex: 0];
    }
    return nil;
}

static NSString *get_element_value(NSXMLElement *element,NSString* name)
{
    NSXMLElement *child = get_child_element(element,name);
    // TouchXML will break up an element containing HTML such as the 'joined the topic...' status
    // items into multiple child elements, so I join them as a single text string
#if TARGET_OS_IPHONE
    if (child) {
        if ([child childCount] > 0) {
            NSArray *elements = [child children];
            NSMutableString *fulltext = [NSMutableString string];
            for (NSXMLElement *elem in elements) {
                NSString *text = [elem stringValue];
                if (nil != text) {
                    [fulltext appendString:text];
                }
            }
            return fulltext;
        }
        else {
            return [child stringValue];
        }
    }
    return nil;
#else
    return child ? [child stringValue] : nil;
#endif
}

@implementation YazzemUser

@synthesize user_id = _user_id;
@synthesize username = _username;
@synthesize full_name = _full_name;
@synthesize bio = _bio;
@synthesize website = _website;
@synthesize location = _location;
@synthesize profile_photo = _profile_photo;
@synthesize reg_date = _reg_date;

- (id)initWithElement: (NSXMLElement*)element
{
    if (self = [super init]) {
        self.user_id = get_element_value(element, @"user_id");
        self.username = get_element_value(element, @"username");
        self.full_name = get_element_value(element, @"full_name");
        self.bio = get_element_value(element, @"bio");
        self.website = get_element_value(element, @"website");
        self.location = get_element_value(element, @"location");
        self.profile_photo = get_element_value(element, @"profile_photo");
        self.reg_date = get_element_value(element, @"reg_date");
    }
    return self;
}

+ (YazzemUser*)UserWithElement: (NSXMLElement*)element
{
    return [[[YazzemUser alloc] initWithElement:element] autorelease];
}

@end

@implementation YazzemStatus

@synthesize text = _text;
@synthesize status_id = _status_id;
@synthesize from = _from;
@synthesize type = _type;
@synthesize date = _date;
@synthesize user = _user;

- (id)initWithElement: (NSXMLElement*)element
{
    if (self = [super init]) {
        self.text = get_element_value(element, @"text");
        self.status_id = get_element_value(element, @"id");
        self.from = get_element_value(element, @"from");
        self.type = get_element_value(element, @"type");
        self.date = get_element_value(element, @"submit_date");

        // don't create an uninitialized user object if none is present
        NSXMLElement *user = get_child_element(element, @"user");
        if (nil != user) {
            self.user = [YazzemUser UserWithElement: user];
        }
    }
    return self;
}

+ (YazzemStatus*)StatusWithElement: (NSXMLElement*)element
{
    return [[[YazzemStatus alloc] initWithElement:element] autorelease];
}


@end

@implementation YazzemTopic

@synthesize topic_name = _topic_name;
@synthesize topic_id = _topic_id;
@synthesize type = _type;
@synthesize description = _description;
@synthesize keywords = _keywords;
@synthesize date = _date;
@synthesize user = _user;

- (id)initWithElement: (NSXMLElement*)element
{
    if (self = [super init]) {
        self.topic_name = get_element_value(element, @"topic_name");
        self.topic_id = get_element_value(element, @"topic_id");
        self.type = get_element_value(element, @"type");
        self.description = get_element_value(element, @"description");
        self.keywords = get_element_value(element, @"keywords");
        self.date = get_element_value(element, @"created_date");
        
        // don't create an uninitialized user object if none is present
        NSXMLElement *user = get_child_element(element, @"creator");
        if (nil != user) {
            self.user = [YazzemUser UserWithElement: user];
        }
    }
    return self;
}

+ (YazzemTopic*)TopicWithElement: (NSXMLElement*)element
{
    return [[[YazzemTopic alloc] initWithElement:element] autorelease];
}

@end

@implementation YazzemMessage

@synthesize message_id = _message_id;
@synthesize text = _text;
@synthesize subject = _subject;
@synthesize status = _status;
@synthesize sender = _sender;
@synthesize sentto = _sentto;

- (id)initWithElement: (NSXMLElement*)element
{
    if (self = [super init]) {
        self.message_id = get_element_value(element, @"id");
        self.text = get_element_value(element, @"text");
        self.subject = get_element_value(element, @"subject");
        self.status = get_element_value(element, @"status");

        // don't create an uninitialized user object if none is present
        NSXMLElement *user = get_child_element(element, @"sender");
        if (nil != user) {
            self.sender = [YazzemUser UserWithElement:user];
        }
        else if (nil != (user = get_child_element(element, @"sent_to"))) {
            self.sentto = [YazzemUser UserWithElement:user];
        }
    }
    return self;
}

+ (YazzemMessage*)MessageWithElement: (NSXMLElement*)element
{
    return [[[YazzemMessage alloc] initWithElement:element] autorelease];
}

@end

@implementation YazzemComment

@synthesize comment_id = _comment_id;
@synthesize text = _text;
@synthesize date = _date;
@synthesize user = _user;

- (id)initWithElement: (NSXMLElement*)element
{
    if (self = [super init]) {
        self.comment_id = get_element_value(element, @"id");
        self.text = get_element_value(element, @"text");
        self.date = get_element_value(element, @"submit_date");
        
        // don't create an uninitialized user object if none is present
        NSXMLElement *userElement = get_child_element(element, @"user");
        if (nil != userElement) {
            self.user = [YazzemUser UserWithElement:userElement];
        }
        else {
            if ((userElement = get_child_element(element, @"poster")) != nil) {
                self.user = [YazzemUser UserWithElement:userElement];
            }
        }
    }
    return self;
}

+ (YazzemComment*)CommentWithElement: (NSXMLElement*)element
{
    return [[[YazzemComment alloc] initWithElement:element] autorelease];
}

@end

static Yazzem *_shared_api = nil;

@implementation Yazzem

@synthesize delegate = _delegate;

@synthesize appname = _appname;

@synthesize username = _username;
@synthesize password = _password;
@synthesize endpoint = _endpoint;

@synthesize method = _method;
@synthesize params = _params;

+ (Yazzem*)Shared
{
    if (nil == _shared_api) {
        _shared_api = [[Yazzem alloc] init];
    }
    return _shared_api;
}

- (id) init
{
    if (self = [super init]) {
        self.endpoint = @"http://www.yazzem.com/api";
    }
    return self;
}

- (void)dealloc
{
    // make sure no connection is in progress or we will crash when we release it
    //[_connection cancel];

    //self.connection = nil;
    self.username = nil;
    self.password = nil;
    self.endpoint = nil;
    self.method = nil;
    self.params = nil;
    self.appname = nil;
    [super dealloc];
}

#pragma mark private methods

- (NSData*)build_post_body: (NSDictionary*)postdata
{
    NSMutableString *str = nil;
    for (id key in postdata) {
        NSString *value = (NSString*)[postdata objectForKey:key];
        NSString *encoded = [value stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
        encoded = [encoded stringByReplacingOccurrencesOfString:@"&" withString:@"%26"];
        encoded = [encoded stringByReplacingOccurrencesOfString:@"'" withString:@"%27"];
        if (nil == str) {
            str = [NSMutableString stringWithFormat:@"%@=%@",key, encoded];
        }
        else {
            [str appendFormat: @"&%@=%@",key, encoded];
        }
    }
    return [str dataUsingEncoding:NSUTF8StringEncoding];
}

- (NSXMLElement*)http_post: (NSMutableDictionary*)postdata
{
    NSURLResponse *response = nil;
    NSError *error = nil;
    NSURL *url = [NSURL URLWithString:[NSString stringWithFormat: @"%@/%@",_endpoint,_method]];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url];
    
    // add username & password to request
    [postdata setObject:_password forKey:@"password"];
    [postdata setObject:_username forKey:@"username"];
    if (nil != _appname) {
        [postdata setObject:_appname forKey:@"from_what"];
    }

    [request setHTTPMethod:@"POST"];
    [request setHTTPBody: [self build_post_body:postdata]];
    
    NSData *data = [NSURLConnection sendSynchronousRequest:request 
                                         returningResponse:&response 
                                                     error:&error];
    //return [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease];
    if (nil != data) {
        NSXMLDocument *doc = [[[NSXMLDocument alloc] initWithData: data
                                                         options: NSXMLDocumentTidyXML 
                                                           error: &error] autorelease];
        return [doc rootElement];
    }
    return nil;
}

- (NSData*)http_get
{
    NSURLResponse *response = nil;
    NSError *error = nil;
    NSURL *url = [NSURL URLWithString:[NSString stringWithFormat: @"%@/%@%@", 
                                        _endpoint, _method,
                                        _params ? _params : @""]];
                                        // make sure we don't see (null)!
    // we should really do asynchronous, but to keep it simple I'm doing a synchronous request
    NSURLRequest *request = [NSURLRequest requestWithURL:url];
    NSData *data = [NSURLConnection sendSynchronousRequest:request 
                                         returningResponse:&response 
                                                     error:&error];
    return data;
}

- (NSXMLDocument*)GetXmlResponse
{
    NSError *error = nil;
    NSData *data = [self http_get];
    if (nil != data) {
        NSXMLDocument *doc = [[NSXMLDocument alloc] initWithData: data
                                                         options: NSXMLDocumentTidyXML 
                                                           error: &error];
        return [doc autorelease];
    }
    return nil;
}

- (NSMutableArray*)MakeArrayOfTopics: (NSXMLElement*)root named: (NSString*)name
{
    NSMutableArray *topics = [NSMutableArray array];
    NSArray *tmp = [root elementsForName: name];
    for (NSXMLElement *elem in tmp) {
        [topics addObject: [YazzemTopic TopicWithElement: elem]];
    }
    return topics;
}

- (NSMutableArray*)MakeArrayOfStatuses: (NSXMLElement*)root named: (NSString*)name
{
    NSMutableArray *stats = [NSMutableArray array];
    NSArray *tmp = [root elementsForName: name];
    for (NSXMLElement *elem in tmp) {
        [stats addObject: [YazzemStatus StatusWithElement: elem]];
    }
    return stats;
}

- (NSMutableArray*)MakeArrayOfComments: (NSXMLElement*)root named: (NSString*)name
{
    NSMutableArray *comments = [NSMutableArray array];
    NSArray *tmp = [root elementsForName: name];
    for (NSXMLElement *elem in tmp) {
        [comments addObject:[YazzemComment CommentWithElement: elem]];
    }
    return comments;
}

- (NSMutableArray*)MakeArrayOfMessages: (NSXMLElement*)root named: (NSString*)name
{
    NSMutableArray *messages = [NSMutableArray array];
    NSArray *tmp = [root elementsForName: name];
    for (NSXMLElement *elem in tmp) {
        [messages addObject:[YazzemMessage MessageWithElement: elem]];
    }
    return messages;
}

- (NSMutableArray*)MakeArrayOfUsers: (NSXMLElement*)root named: (NSString*)name
{
    NSMutableArray *users = [NSMutableArray array];
    NSArray *tmp = [root elementsForName: name];
    for (NSXMLElement *elem in tmp) {
        [users addObject:[YazzemUser UserWithElement: elem]];
    }
    return users;
}

#pragma mark status methods

- (NSMutableArray*)get_status:(NSString*)inMethod count: (int)count
{
    self.method = inMethod;
    self.params = [NSString stringWithFormat: @"?count=%d", count ? count : 20];
    NSXMLDocument *doc = [self GetXmlResponse];
    NSXMLElement *root = [doc rootElement];
    return [self MakeArrayOfStatuses:root named:@"status"];
}

- (NSMutableArray*)all_status: (int)count
{
    return [self get_status:@"all_status" count:count];
}


- (NSMutableArray*)status_status: (int)count
{
    return [self get_status:@"status_status" count:count];
}

- (NSMutableArray*)status_topic: (int)count
{
    return [self get_status:@"status_topic" count: count];
}

// NOTE: before calling this you must set username & password
- (NSMutableArray*)get_timeline: (int)count
{
    self.method = @"get_timeline";
    self.params = [NSString stringWithFormat: @"?username=%@&password=%@&count=%d", 
                   _username, _password, (count ? count : 20)];
    NSXMLDocument *doc = [self GetXmlResponse];
    NSXMLElement *root = [doc rootElement];
    return [self MakeArrayOfStatuses:root named:@"status"];
}

- (NSMutableArray*)get_status_comments: (NSString*)status_id count: (int)count
{
    self.method = @"get_status_comments";
    self.params = [NSString stringWithFormat: @"?status_id=%@&count=%d",
                   status_id,(count ? count : 20)];
    NSXMLDocument *doc = [self GetXmlResponse];
    NSXMLElement *root = [doc rootElement];
    return [self MakeArrayOfComments:root named:@"comment"];
}

#pragma mark topic methods

- (YazzemTopic*)get_topic:(NSString*)topic_id
{
    self.method = @"get_topic";
    self.params = [NSString stringWithFormat:@"?id=%@",topic_id];
    NSXMLDocument *doc = [self GetXmlResponse];
    return [YazzemTopic TopicWithElement: get_child_element([doc rootElement], @"topic")];
}

- (NSMutableArray*)get_topic_comments: (NSString*)topic_id count: (int)count
{
    self.method = @"get_topic_comments";
    self.params = [NSString stringWithFormat: @"?topic_id=%@&count=%d",
                   topic_id,(count ? count : 20)];
    NSXMLDocument *doc = [self GetXmlResponse];
    NSXMLElement *root = [doc rootElement];
    return [self MakeArrayOfComments:root named:@"comment"];
}

- (NSMutableArray*)get_topic_list: (int)count
{
    self.method = @"get_topic_list";
    self.params = [NSString stringWithFormat: @"?count=%d",(count ? count : 20)];
    NSXMLDocument *doc = [self GetXmlResponse];
    NSXMLElement *root = [doc rootElement];
    return [self MakeArrayOfTopics:root named:@"topic"];
}

#pragma mark user methods

- (YazzemUser*)get_user: (NSString*)user_id
{
    self.method = @"get_user";
    self.params = [NSString stringWithFormat:@"?id=%@",user_id];
    NSXMLDocument *doc = [self GetXmlResponse];
    return [YazzemUser UserWithElement: get_child_element([doc rootElement], @"user")];
}

- (NSMutableArray*)get_user_topics: (NSString*)user_id count:(int)count
{
    // NOTE: count is ignored, since get_user_topics doesn't take a count
    self.method = @"get_user_topics";
    self.params = [NSString stringWithFormat: @"?id=%@",user_id];
    NSXMLDocument *doc = [self GetXmlResponse];
    NSXMLElement *root = [doc rootElement];
    return [self MakeArrayOfTopics:root named:@"topics"];
}

- (NSMutableArray*)get_user_joined: (NSString*)user_id count:(int)count
{
    // NOTE: count is ignored, since get_user_topics doesn't take a count
    self.method = @"get_user_joined";
    self.params = [NSString stringWithFormat: @"?id=%@",user_id];
    NSXMLDocument *doc = [self GetXmlResponse];
    NSXMLElement *root = [doc rootElement];
    return [self MakeArrayOfTopics:root named:@"topics"];
}

- (NSMutableArray*)get_user_status: (NSString*)user_id count:(int)count
{
    self.method = @"get_user_status";
    self.params = [NSString stringWithFormat: @"?id=%@&count=%d",user_id,
                                                    (count ? count : 20)];
    NSXMLDocument *doc = [self GetXmlResponse];
    NSXMLElement *root = [doc rootElement];
    return [self MakeArrayOfStatuses:root named:@"status"];
}

- (NSMutableArray*)get_profile_comments: (NSString*)user_id count:(int)count
{
    self.method = @"get_profile_comments";
    self.params = [NSString stringWithFormat: @"?user_id=%@&count=%d",user_id, (count ? count : 20)];
    NSXMLDocument *doc = [self GetXmlResponse];
    NSXMLElement *root = [doc rootElement];
    return [self MakeArrayOfComments:root named:@"comment"];
}

- (NSMutableArray*)get_user_subscribers: (NSString*)user_id count:(int)count
{
    self.method = @"get_user_subscribers";
    self.params = [NSString stringWithFormat: @"?user_id=%@&count=%d",user_id, (count ? count : 20)];
    NSXMLDocument *doc = [self GetXmlResponse];
    NSXMLElement *root = [doc rootElement];
    return [self MakeArrayOfUsers:root named: @"subscriber"];
}

- (NSMutableArray*)get_user_subscriptions: (NSString*)user_id count:(int)count
{
    self.method = @"get_user_subscriptions";
    self.params = [NSString stringWithFormat: @"?user_id=%@&count=%d",user_id, (count ? count : 20)];
    NSXMLDocument *doc = [self GetXmlResponse];
    NSXMLElement *root = [doc rootElement];
    return [self MakeArrayOfUsers:root named: @"subscription"];
}

- (NSString*)get_userID: (NSString*)name
{
    self.method = @"get_userID";
    self.params = [NSString stringWithFormat: @"?username=%@",name];
    NSXMLDocument *doc = [self GetXmlResponse];
    NSXMLElement *root = [doc rootElement];
    return get_element_value(root, @"user_id");
}

#pragma mark message methods

- (NSMutableArray*)get_messages:(int)count
{
    self.method = @"get_messages";
    self.params = [NSString stringWithFormat: @"?username=%@&password=%@&count=%d", _username, _password,
                                                        (count ? count : 20)];
    NSXMLDocument *doc = [self GetXmlResponse];
    NSXMLElement *root = [doc rootElement];
    return [self MakeArrayOfMessages:root named:@"message"];
}

- (NSMutableArray*)get_messages_read:(int)count
{
    self.method = @"get_messages_read";
    self.params = [NSString stringWithFormat: @"?username=%@&password=%@&count=%d", _username, _password,
                                                        (count ? count : 20)];
    NSXMLDocument *doc = [self GetXmlResponse];
    NSXMLElement *root = [doc rootElement];
    return [self MakeArrayOfMessages:root named:@"message"];
}

- (NSMutableArray*)get_messages_unread:(int)count
{
    self.method = @"get_messages_unread";
    self.params = [NSString stringWithFormat: @"?username=%@&password=%@&count=%d", _username, _password,
                                                        (count ? count : 20)];
    NSXMLDocument *doc = [self GetXmlResponse];
    NSXMLElement *root = [doc rootElement];
    return [self MakeArrayOfMessages:root named:@"message"];
}

- (NSMutableArray*)get_messages_sent: (int)count
{
    self.method = @"get_messages_sent";
    self.params = [NSString stringWithFormat: @"?username=%@&password=%@&count=%d", _username, _password,
                                                        (count ? count : 20)];
    NSXMLDocument *doc = [self GetXmlResponse];
    NSXMLElement *root = [doc rootElement];
    return [self MakeArrayOfMessages:root named:@"message"];
}

#pragma mark post and modify data methods

- (YazzemStatus*) status_new: (NSString*)text
{
    self.method = @"status_new";
    NSMutableDictionary* statusParams = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                        text, @"user_status", nil];
    NSXMLElement *root = [self http_post: statusParams];
    return [YazzemStatus StatusWithElement: get_child_element(root, @"status")];
}

- (YazzemComment*) status_comment_new: (NSString*)status_id comment: (NSString*)text
{
    self.method = @"status_comment_new";
    NSXMLElement *root = [self http_post:
                      [NSMutableDictionary dictionaryWithObjectsAndKeys: status_id, @"status_id",
                       text, @"comment", nil]];
    return [YazzemComment CommentWithElement:get_child_element(root, @"comment")];
}

- (YazzemTopic*) topic_create: (NSString*)name 
                  description: (NSString*)description 
                     keywords: (NSString*)keywords
                    is_public: (BOOL)is_public
{
    self.method = @"topic_create";
    NSXMLElement *root = [self http_post:
                      [NSMutableDictionary dictionaryWithObjectsAndKeys: name, @"topic_name",
                       (is_public ? @"true" : @"false"), @"type", description, @"description",
                       keywords, @"keywords",nil]];
    return [YazzemTopic TopicWithElement:get_child_element(root, @"topic")];
}

- (YazzemTopic*) topic_join: (NSString*)topic_id
{
    self.method = @"topic_join";
    NSXMLElement *root = [self http_post: [NSMutableDictionary dictionaryWithObjectsAndKeys:topic_id, @"topic_id",nil]];
    return [YazzemTopic TopicWithElement:get_child_element(root, @"topic")];
}

- (YazzemTopic*) topic_leave: (NSString*)topic_id
{
    self.method = @"topic_leave";
    NSXMLElement *root = [self http_post: [NSMutableDictionary dictionaryWithObjectsAndKeys:topic_id, @"topic_id",nil]];
    return [YazzemTopic TopicWithElement:get_child_element(root, @"topic")];
}

- (YazzemComment*) topic_comment_new: (NSString*)topic_id comment: (NSString*)text
{
    self.method = @"topic_comment_new";
    NSXMLElement *root = [self http_post: 
                      [NSMutableDictionary dictionaryWithObjectsAndKeys:topic_id, @"topic_id", text, @"comment", nil]];
    return [YazzemComment CommentWithElement:get_child_element(root, @"topic")];
}

- (YazzemUser*) subscribe: (NSString*)user_id
{
    self.method = @"subscribe";
    NSXMLElement *root = [self http_post: [NSMutableDictionary dictionaryWithObjectsAndKeys: user_id, @"profile_id", nil]];
    return [YazzemUser UserWithElement: get_child_element(root, @"user")];
}

- (YazzemUser*) unsubscribe: (NSString*)user_id
{
    self.method = @"unsubscribe";
    NSXMLElement *root = [self http_post: [NSMutableDictionary dictionaryWithObjectsAndKeys: user_id, @"profile_id", nil]];
    return [YazzemUser UserWithElement: get_child_element(root, @"user")];
}

- (YazzemComment*) profile_comment_new: (NSString*)user_id comment: (NSString*)text
{
    self.method = @"profile_comment_new";
    NSXMLElement *root = [self http_post: 
                      [NSMutableDictionary dictionaryWithObjectsAndKeys: user_id, @"profile_id", text, @"comment", nil]];
    return [YazzemComment CommentWithElement:get_child_element(root, @"comment")];
}

- (YazzemMessage*) message_new: (NSString*)to_id
             subject: (NSString*)subject
             message: (NSString*)message
            is_reply: (BOOL)is_reply
{
    self.method = @"message_new";
    NSXMLElement *root = [self http_post: 
                      [NSMutableDictionary dictionaryWithObjectsAndKeys: to_id, @"to_id", message, @"message",
                       (is_reply ? @"true" : @"false"), @"reply", subject, @"subject", nil]];
    return [YazzemMessage MessageWithElement:get_child_element(root, @"message")];
}

- (void) message_read: (NSString*)message_id
{
    self.method = @"message_read";
    [self http_post: [NSMutableDictionary dictionaryWithObjectsAndKeys: message_id, @"message_id", nil]];
}

- (void) message_unread: (NSString*)message_id
{
    self.method = @"message_unread";
    [self http_post: [NSMutableDictionary dictionaryWithObjectsAndKeys: message_id, @"message_id", nil]];
}


@end
