//
//  RootViewController.m
//  YazzemTouch
//
//  Created by Mike Cohen on 2/26/10.
//  Copyright MC Development 2010. All rights reserved.
//

#import "RootViewController.h"
#import "StatusDetailController.h"
#import "Yazzem.h"

@implementation RootViewController

@synthesize items = _items;
static Yazzem *api;

- (IBAction)reload:(id)sender
{
    NSUserDefaults *def = [NSUserDefaults standardUserDefaults];
    api.username = [def stringForKey:@"username"];
    api.password = [def stringForKey:@"password"];

    if ((nil != api.username) && (nil != api.password)) {
        self.items = [api get_timeline:20];
    }
    else {
        self.items = [api all_status:20];
    }
}

#pragma mark -
#pragma mark View lifecycle

/* */
- (void)viewDidLoad {
    [super viewDidLoad];
    api = [Yazzem Shared];
    

    _titles = [[NSArray alloc] initWithObjects: @"Log in", 
                                                @"List Topics",
                                                @"Update status", 
                                                @"Create Topic", nil];

    _views = [[NSArray alloc] initWithObjects:  @"LoginViewController", 
                                                @"TopicListViewController",
                                                @"UpdateViewController",
                                                @"NewTopicViewController", nil];

    [self reload: nil];

}
/* */

/* */
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
}
/* */
/*
- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
}
*/
/*
- (void)viewWillDisappear:(BOOL)animated {
	[super viewWillDisappear:animated];
}
*/
/*
- (void)viewDidDisappear:(BOOL)animated {
	[super viewDidDisappear:animated];
}
*/

/*
 // Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	// Return YES for supported orientations.
	return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
 */


#pragma mark -
#pragma mark Table view data source

// Customize the number of sections in the table view.
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 2;
}


// Customize the number of rows in the table view.
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return (0 == section) ? [_titles count] : [_items count];
}


// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
    }
    
	// Configure the cell.
    if (0 == indexPath.section) {
        cell.textLabel.text = [_titles objectAtIndex: indexPath.row];
    }
    else {
        YazzemStatus *status = [_items objectAtIndex: indexPath.row];
        YazzemUser* user = status.user;
        cell.textLabel.text = [NSString stringWithFormat: @"%@ - %@",user.username,status.text];
    }

    return cell;
}


/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/


/*
// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source.
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
    }   
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view.
    }   
}
*/


/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
}
*/


/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/


#pragma mark -
#pragma mark Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    if (0 == indexPath.section) {
        NSString *viewName = [_views objectAtIndex:indexPath.row];
        Class viewclass = NSClassFromString(viewName);
        if (nil != viewclass) {
            UIViewController *detailView = [[viewclass alloc] initWithNibName:viewName bundle:nil];
            [self.navigationController pushViewController:detailView animated:YES];
            [detailView release];
        }
    }
    else {
        // display status details
        StatusDetailController *detail = [[StatusDetailController alloc] initWithNibName:@"StatusDetailController" bundle:nil];
        detail.status = [_items objectAtIndex:indexPath.row];
        [self.navigationController pushViewController:detail animated:YES];
        [detail release];
    }
}


#pragma mark -
#pragma mark Memory management

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Relinquish ownership any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
    // Relinquish ownership of anything that can be recreated in viewDidLoad or on demand.
    // For example: self.myOutlet = nil;
}


- (void)dealloc {
    self.items = nil;
    [_titles release];
    [_views release];
    [super dealloc];
}


@end

